function f=MMnorm(roots,a,model) 
% f = MMnorm(roo,a,model)
% returns the normalization coefficients for the eigenfunction of the 
% diffusion equation in the given substrate
% 
% input: 

% roots: roots of the eigenfunction of the diffusion equation for the given
% restriction
% a: size of restriction
% model: model id: 0 for sphere, 1 for cylinder, 2 for parallel planes
%
% author: Andrada Ianus (a.ianus.11@ucl.ac.uk), Ivana Drobnjak (i.drobnjak@ucl.ac.uk), Daniel
% C. Alexander (d.alexander@ucl.ac.uk)
%
SMALL=1E-5;
if model==0
    n = roots(1);
    m = roots(2); % upper index of the associated legendre polynomials P_n^m
    alpha = roots(4);
    if n ==0 && alpha <SMALL
       f = (a^3*4*pi/3)^(-1/2);        
    else
    f1=(a^3*pi*((spherical_besselj(n,alpha)).^2-spherical_besselj(n-1,alpha).*...
    spherical_besselj(n+1,alpha))).^(-1/2);
    f = f1*(factorial(n+m)*2/(2*n+1)/(factorial(n-m)))^(-1/2);
    end
elseif model==1
    n = roots(1);
    alpha = roots(3);
    if n==0
        k=1;
    else
        k=2;
    end
    tmp1=k/(pi*a^2*(besselj(n,alpha)).^2);
    if n==0 && alpha<SMALL
        tmp2=1;
    else
        tmp2=alpha^2/(alpha^2-n^2);
    end
    f=(tmp1*tmp2)^(1/2);
elseif model==2
    n = roots(1);
    alpha = roots(2);
    if alpha == 0    
    % add the case for eigenval 0
    f = 1./sqrt(a*2);
    else
        f=sqrt(1./(a*(1+(-1)^n*sin(2*alpha)/(2*alpha))));
    end
else
    error('Model index not specified. 0 for sphere, 1 for cylinder, 2 for planar.')
end
    