function R=MatrixR(tau,a,dimA,roo,model,D)
% R = MatrixR(tau,a,dimA,roo,model,D)
% returns the matrix R from the Matrix Formulation of the diffusion signal
% 
% input: 
% tau: discretisation time in s
% a: size of restriction
% dimA: dimention of matrix R
% roo: roots of the eigenfunction of the diffusion equation for the given
% restriction
% model: model id: 0 for sphere, 1 for cylinder, 2 for parallel planes
% D - diffusivity constant in SI
%
% author: Andrada Ianus (a.ianus.11@ucl.ac.uk), Ivana Drobnjak (i.drobnjak@ucl.ac.uk), Daniel
% C. Alexander (d.alexander@ucl.ac.uk)
%


R=zeros(dimA,dimA);
for m=1:dimA
  if model==2 % parallel planes
     alpha = roo(m,2);
  elseif model == 1 % cylinder
     alpha = roo(m,3);
  elseif model == 0 % spheres
      alpha = roo(m,4);       
  end
  R(m,m)= exp(-alpha^2*tau*D/(a^2));
end