function [E, J] = SynthMeasTortZeppelinCylinder(params,protocol,params_deriv)
% function [E J] = SynthMeasTortZeppelinCylinder(model, protocol, params_deriv)
% returns the diffusion signal and signal jacobian for the given protocol
% and TortZeppelinCylinder tissue model
%
% author: Andrada Ianus (a.ianus.11@ucl.ac.uk)
%
% model parameters are: f, dPar, dPerp, R, theta, phi
% $Id$
fun_name1 = ['Zeppelin_' protocol.pulseseq];
fun_name2 = ['Cylinder_' protocol.pulseseq];

fun1 = str2func(fun_name1);
fun2 = str2func(fun_name2);
f = params(1); % volume fraction of cylinder compartment

params_zep = [params(2) params(2)*(1-f) params(4:5)]; % turtuosity constraint
params_cyl = params(2:end); % parameters for cylinder

if nargout == 1           
        E = (1-f)*fun1(params_zep,protocol)+f*fun2(params_cyl,protocol);
else 
    if nargin <3
        params_deriv = ones(size(params));   
    end
    params_deriv_zep = [params_deriv(2) (params_deriv(1) || params_deriv(2)) params_deriv(4:5)];
    params_deriv_cyl = params_deriv(2:end);
    
    [E1 J1] = fun1(params_zep,protocol,params_deriv_zep);
    [E2 J2] = fun2(params_cyl,protocol,params_deriv_cyl);
    E = (1-f)*E1+f*E2;
    J = zeros(length(E),nnz(params_deriv));
    it = 0;
    if params_deriv(1) it = it+1; J(:,it) = E2 - E1 + (1-f)*J1(:,2)*(-params_zep(1)); end % volume fraction, including turtuosity term
    if params_deriv(2) it = it+1; J(:,it) = (1-f)*J1(:,1) + f*J2(:,1) + (1-f)^2*J1(:,2) ; end % Dpar, including turtuosity term
    if params_deriv(3) it = it+1; J(:,it) = f*J2(:,2); end % radius
    if params_deriv(4) it = it+1; J(:,it) = (1-f)*J1(:,3) + f*J2(:,3); end % theta
    if params_deriv(5) it = it+1; J(:,it) = (1-f)*J1(:,4) + f*J2(:,4); end % phi 
    
   
end
end