function [E, J] = SynthMeasTortZeppelinCylinderBall(params,protocol,params_deriv)
% function [E J] = SynthMeasTortZeppelinCylinderBall(model, protocol, params_deriv)
% returns the diffusion signal and signal jacobian for the given protocol
% and TortZeppelinCylinderBall tissue model
%
% author: Andrada Ianus (a.ianus.11@ucl.ac.uk)
% $Id$

fun_name1 = ['Zeppelin_' protocol.pulseseq];
fun_name2 = ['Cylinder_' protocol.pulseseq];
fun_name3 = ['Ball_' protocol.pulseseq];

fun1 = str2func(fun_name1);
fun2 = str2func(fun_name2);
fun3 = str2func(fun_name3);
ficvf = params(1); % volume fraction of cylinder compartment as fraction of anisotropic part
di = params(2);
rad = params(3);
fiso = params(4);
diso = params(5);
theta = params(6);
phi = params(7);

params_zep = [di (1-ficvf)*di theta phi]; % turtuosity constraint
params_cyl = [di rad theta phi]; % parameters for cylinder
params_ball = [diso]; % parameters for ball


if nargout == 1           
        E = (1-ficvf)*(1-fiso)*fun1(params_zep,protocol)+ficvf*(1-fiso)*fun2(params_cyl,protocol)+ fiso*fun3(params_ball,protocol);
else 
    if nargin <3
        params_deriv = ones(size(params));
    end
    params_deriv_zep = [params_deriv(2) (params_deriv(1) || params_deriv(2)) params_deriv(6:7)];
    params_deriv_cyl = [params_deriv(2) params_deriv(3) params_deriv(6:7)];
    params_deriv_ball = [params_deriv(5)];
    
    [E1 J1] = fun1(params_zep,protocol,params_deriv_zep);
    [E2 J2] = fun2(params_cyl,protocol,params_deriv_cyl);
    [E3 J3] = fun2(params_ball,protocol,params_deriv_ball);
     E = (1-ficvf)*(1-fiso)*E1+ficvf*(1-fiso)*E2+ fiso*E3;
    J = zeros(length(E),nnz(params_deriv));
    it = 0;
    if params_deriv(1) % ficvf volume fraction, including turtuosity term
        it = it+1; 
        J(:,it) = (1-fiso)*E2 - (1-fiso)*E1 + (1-fiso)*(1-ficvf)*J1(:,2)*(-params_zep(1)); 
    end 
    if params_deriv(2) % di, including turtuosity term
        it = it+1; 
        J(:,it) = (1-ficvf)*(1-fiso)*J1(:,1) + ficvf*(1-fiso)*J2(:,1) + (1-ficvf)*(1-fiso)*J1(:,2) ; 
    end 
    if params_deriv(3) % radius
        it = it+1;
        J(:,it) = ficvf*(1-fiso)*J2(:,2); 
    end 
    if params_deriv(4) % fiso
        it = it+1; 
        J(:,it) = -(1-ficvf)*E1 - ficvf* E2 + E3; 
    end 
    if params_deriv(5) % diso
        it = it+1; 
        J(:,it) = fiso*J3(:,1); 
    end 
    if params_deriv(6) % theta
        it = it+1; 
        J(:,it) = (1-ficvf)*(1-fiso)*J1(:,3) + ficvf*(1-fiso)*J2(:,3); 
    end 
    if params_deriv(7) % phi
        it = it+1; 
        J(:,it) = (1-ficvf)*(1-fiso)*J1(:,4) + ficvf*(1-fiso)*J2(:,4); 
    end  
    
   
end
end