function [E, J] = SynthMeasZeppelinCylinder(params,protocol,params_deriv)
% function [E J] = SynthMeasZeppelinCylinder(model, protocol, params_deriv)
% returns the diffusion signal and signal jacobian for the given protocol
% and ZeppelinCylinder tissue model
%
% author: Andrada Ianus (a.ianus.11@ucl.ac.uk)

fun_name1 = ['Zeppelin_' protocol.pulseseq];
fun_name2 = ['Cylinder_' protocol.pulseseq];

fun1 = str2func(fun_name1);
fun2 = str2func(fun_name2);
index1 = [2 3 5 6]; % parameters for zeppelin
index2 = [2 4 5 6]; % parameters for cylinder
f = params(1); % volume fraction of cylinder compartment

if nargout == 1           
        E = (1-f)*fun1(params(index1),protocol)+f*fun2(params(index2),protocol);
else 
    if nargin <3
        params_deriv = ones(size(params));
    end
    [E1 J1] = fun1(params(index1),protocol,params_deriv(index1));
    [E2 J2] = fun2(params(index2),protocol,params_deriv(index2));
    E = (1-f)*E1+f*E2;
    J = zeros(length(E),nnz(params_deriv));
    it = 0;
    if params_deriv(1) it = it+1; J(:,it) = E2 - E1; end % volume fraction
    if params_deriv(2) it = it+1; J(:,it) = (1-f)*J1(:,1) + f*J2(:,1); end % Dpar
    if params_deriv(3) it = it+1; J(:,it) = (1-f)*J1(:,2); end % Dperp
    if params_deriv(4) it = it+1; J(:,it) = f*J2(:,2); end % radius
    if params_deriv(5) it = it+1; J(:,it) = (1-f)*J1(:,3) + f*J2(:,3); end % theta
    if params_deriv(6) it = it+1; J(:,it) = (1-f)*J1(:,4) + f*J2(:,4); end % phi 
    
   
end
end